@@ -26,6 +26,7 @@
 
 #include "loader.h"
 #include "debug_utils.h"
+#include "unknown_function_handling.h"
 #include "wsi.h"
 
 void *trampoline_get_proc_addr(struct loader_instance *inst, const char *funcName) {
@@ -251,17 +252,18 @@ void *trampoline_get_proc_addr(struct loader_instance *inst, const char *funcNam
 
     // Instance extensions
     void *addr;
-    if (debug_utils_InstanceGpa(inst, funcName, &addr)) return addr;
+    if (debug_extensions_InstanceGpa(inst, funcName, &addr)) return addr;
 
     if (wsi_swapchain_instance_gpa(inst, funcName, &addr)) return addr;
 
     if (extension_instance_gpa(inst, funcName, &addr)) return addr;
 
     // Unknown physical device extensions
-    if (loader_phys_dev_ext_gpa(inst, funcName, true, &addr, NULL)) return addr;
+    addr = loader_phys_dev_ext_gpa_tramp(inst, funcName);
+    if (NULL != addr) return addr;
 
     // Unknown device extensions
-    addr = loader_dev_ext_gpa(inst, funcName);
+    addr = loader_dev_ext_gpa_tramp(inst, funcName);
     return addr;
 }
 
