@@ -1,8 +1,8 @@
 /*
  *
- * Copyright (c) 2014-2021 The Khronos Group Inc.
- * Copyright (c) 2014-2021 Valve Corporation
- * Copyright (c) 2014-2021 LunarG, Inc.
+ * Copyright (c) 2014-2022 The Khronos Group Inc.
+ * Copyright (c) 2014-2022 Valve Corporation
+ * Copyright (c) 2014-2022 LunarG, Inc.
  * Copyright (C) 2015 Google Inc.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
@@ -29,13 +29,16 @@
 
 #pragma once
 
-#include "vk_loader_platform.h"
+#include "vulkan/vk_platform.h"
+#include "vulkan/vk_sdk_platform.h"
+#include <vulkan/vulkan.h>
+#include <vulkan/vk_layer.h>
+#include <vulkan/vk_icd.h>
 
-enum layer_type_flags {
-    VK_LAYER_TYPE_FLAG_INSTANCE_LAYER = 0x1,  // If not set, indicates Device layer
-    VK_LAYER_TYPE_FLAG_EXPLICIT_LAYER = 0x2,  // If not set, indicates Implicit layer
-    VK_LAYER_TYPE_FLAG_META_LAYER = 0x4,      // If not set, indicates standard layer
-};
+#include "vk_loader_platform.h"
+#include "vk_loader_layer.h"
+#include "vk_layer_dispatch_table.h"
+#include "vk_loader_extensions.h"
 
 typedef enum VkStringErrorFlagBits {
     VK_STRING_ERROR_NONE = 0x00000000,
@@ -45,15 +48,15 @@ typedef enum VkStringErrorFlagBits {
 } VkStringErrorFlagBits;
 typedef VkFlags VkStringErrorFlags;
 
-static const int MaxLoaderStringLength = 256;
-static const char UTF8_ONE_BYTE_CODE = 0xC0;
-static const char UTF8_ONE_BYTE_MASK = 0xE0;
-static const char UTF8_TWO_BYTE_CODE = 0xE0;
-static const char UTF8_TWO_BYTE_MASK = 0xF0;
-static const char UTF8_THREE_BYTE_CODE = 0xF0;
-static const char UTF8_THREE_BYTE_MASK = 0xF8;
-static const char UTF8_DATA_BYTE_CODE = 0x80;
-static const char UTF8_DATA_BYTE_MASK = 0xC0;
+static const int MaxLoaderStringLength = 256;           // 0xFF;
+static const unsigned char UTF8_ONE_BYTE_CODE = 192;    // 0xC0;
+static const unsigned char UTF8_ONE_BYTE_MASK = 224;    // 0xE0;
+static const unsigned char UTF8_TWO_BYTE_CODE = 224;    // 0xE0;
+static const unsigned char UTF8_TWO_BYTE_MASK = 240;    // 0xF0;
+static const unsigned char UTF8_THREE_BYTE_CODE = 240;  // 0xF0;
+static const unsigned char UTF8_THREE_BYTE_MASK = 248;  // 0xF8;
+static const unsigned char UTF8_DATA_BYTE_CODE = 128;   // 0x80;
+static const unsigned char UTF8_DATA_BYTE_MASK = 192;   // 0xC0;
 
 // form of all dynamic lists/arrays
 // only the list element should be changed
@@ -104,12 +107,36 @@ struct loader_override_expiration {
     uint8_t minute;
 };
 
+// This structure is used to store the json file version in a more manageable way.
+typedef struct {
+    uint16_t major;
+    uint16_t minor;
+    uint16_t patch;
+} loader_api_version;
+
+// Enumeration used to clearly identify reason for library load failure.
+enum loader_layer_library_status {
+    LOADER_LAYER_LIB_NOT_LOADED = 0,
+
+    LOADER_LAYER_LIB_SUCCESS_LOADED = 1,
+
+    LOADER_LAYER_LIB_ERROR_WRONG_BIT_TYPE = 20,
+    LOADER_LAYER_LIB_ERROR_FAILED_TO_LOAD = 21,
+};
+
+enum layer_type_flags {
+    VK_LAYER_TYPE_FLAG_INSTANCE_LAYER = 0x1,  // If not set, indicates Device layer
+    VK_LAYER_TYPE_FLAG_EXPLICIT_LAYER = 0x2,  // If not set, indicates Implicit layer
+    VK_LAYER_TYPE_FLAG_META_LAYER = 0x4,      // If not set, indicates standard layer
+};
+
 struct loader_layer_properties {
     VkLayerProperties info;
     enum layer_type_flags type_flags;
     uint32_t interface_version;  // PFN_vkNegotiateLoaderLayerInterfaceVersion
     char manifest_file_name[MAX_STRING_SIZE];
     char lib_name[MAX_STRING_SIZE];
+    enum loader_layer_library_status lib_status;
     loader_platform_dl_handle lib_handle;
     struct loader_layer_functions functions;
     struct loader_extension_list instance_extension_list;
@@ -141,29 +168,11 @@ struct loader_layer_list {
     struct loader_layer_properties *list;
 };
 
-struct loader_dispatch_hash_list {
-    size_t capacity;
-    uint32_t count;
-    uint32_t *index;  // index into the dev_ext dispatch table
-};
-
-// loader_dispatch_hash_entry and loader_dev_ext_dispatch_table.dev_ext have
-// one to one correspondence; one loader_dispatch_hash_entry for one dev_ext
-// dispatch entry.
-// Also have a one to one correspondence with functions in dev_ext_trampoline.c
-struct loader_dispatch_hash_entry {
-    char *func_name;
-    struct loader_dispatch_hash_list list;  // to handle hashing collisions
-};
-
 typedef VkResult(VKAPI_PTR *PFN_vkDevExt)(VkDevice device);
-struct loader_dev_ext_dispatch_table {
-    PFN_vkDevExt dev_ext[MAX_NUM_UNKNOWN_EXTS];
-};
 
 struct loader_dev_dispatch_table {
     VkLayerDispatchTable core_dispatch;
-    struct loader_dev_ext_dispatch_table ext_dispatch;
+    PFN_vkDevExt ext_dispatch[MAX_NUM_UNKNOWN_EXTS];
 };
 
 // per CreateDevice structure
@@ -210,6 +219,7 @@ struct loader_icd_term {
     struct loader_icd_term *next;
 
     PFN_PhysDevExt phys_dev_ext[MAX_NUM_UNKNOWN_EXTS];
+    bool supports_get_dev_prop_2;
 };
 
 // Per ICD library structure
@@ -235,8 +245,7 @@ struct loader_instance {
     uint64_t magic;                               // Should be LOADER_MAGIC_NUMBER
 
     // Vulkan API version the app is intending to use.
-    uint16_t app_api_major_version;
-    uint16_t app_api_minor_version;
+    loader_api_version app_api_version;
 
     // We need to manually track physical devices over time.  If the user
     // re-queries the information, we don't want to delete old data or
@@ -252,8 +261,6 @@ struct loader_instance {
     // device stored internal to the public structures.
     uint32_t phys_dev_group_count_term;
     struct VkPhysicalDeviceGroupProperties **phys_dev_groups_term;
-    uint32_t phys_dev_group_count_tramp;
-    struct VkPhysicalDeviceGroupProperties **phys_dev_groups_tramp;
 
     struct loader_instance *next;
 
@@ -261,8 +268,10 @@ struct loader_instance {
     struct loader_icd_term *icd_terms;
     struct loader_icd_tramp_list icd_tramp_list;
 
-    struct loader_dispatch_hash_entry dev_ext_disp_hash[MAX_NUM_UNKNOWN_EXTS];
-    struct loader_dispatch_hash_entry phys_dev_ext_disp_hash[MAX_NUM_UNKNOWN_EXTS];
+    uint32_t dev_ext_disp_function_count;
+    char *dev_ext_disp_functions[MAX_NUM_UNKNOWN_EXTS];
+    uint32_t phys_dev_ext_disp_function_count;
+    char *phys_dev_ext_disp_functions[MAX_NUM_UNKNOWN_EXTS];
 
     struct loader_msg_callback_map_entry *icd_msg_callback_map;
 
@@ -280,18 +289,20 @@ struct loader_instance {
     VkInstance instance;  // layers/ICD instance returned to trampoline
 
     struct loader_extension_list ext_list;  // icds and loaders extensions
-    union loader_instance_extension_enables enabled_known_extensions;
+    struct loader_instance_extension_enables enabled_known_extensions;
 
+    // Stores debug callbacks - used in the log
     VkLayerDbgFunctionNode *DbgFunctionHead;
-    uint32_t num_tmp_report_callbacks;
-    VkDebugReportCallbackCreateInfoEXT *tmp_report_create_infos;
-    VkDebugReportCallbackEXT *tmp_report_callbacks;
-    uint32_t num_tmp_messengers;
-    VkDebugUtilsMessengerCreateInfoEXT *tmp_messenger_create_infos;
-    VkDebugUtilsMessengerEXT *tmp_messengers;
+
+    // Stores the debug callbacks set during instance creation
+    // These are kept separate because they aren't to be used outside of instance creation and destruction
+    // So they are swapped out at the end of instance creation and swapped in at instance destruction
+    VkLayerDbgFunctionNode *InstanceCreationDeletionDebugFunctionHead;
 
     VkAllocationCallbacks alloc_callbacks;
 
+    bool portability_enumeration_enabled;
+
     bool wsi_surface_enabled;
 #ifdef VK_USE_PLATFORM_WIN32_KHR
     bool wsi_win32_surface_enabled;
@@ -335,6 +346,8 @@ struct loader_instance {
 #endif
     bool wsi_display_enabled;
     bool wsi_display_props2_enabled;
+    bool create_terminator_invalid_extension;
+    bool supports_get_dev_prop_2;
 };
 
 // VkPhysicalDevice requires special treatment by loader.  Firstly, terminator
@@ -369,6 +382,44 @@ struct loader_physical_device_term {
     VkPhysicalDevice phys_dev;  // object from ICD
 };
 
+#ifdef LOADER_ENABLE_LINUX_SORT
+// Structure for storing the relevent device information for selecting a device.
+// NOTE: Needs to be defined here so we can store this content in the term structrue
+//       for quicker sorting.
+struct LinuxSortedDeviceInfo {
+    // Associated Vulkan Physical Device
+    VkPhysicalDevice physical_device;
+    bool default_device;
+
+    // Loader specific items about the driver providing support for this physical device
+    uint32_t icd_index;
+    struct loader_icd_term *icd_term;
+
+    // Some generic device properties
+    VkPhysicalDeviceType device_type;
+    char device_name[VK_MAX_PHYSICAL_DEVICE_NAME_SIZE];
+    uint32_t vendor_id;
+    uint32_t device_id;
+
+    // PCI information on this device
+    bool has_pci_bus_info;
+    uint32_t pci_domain;
+    uint32_t pci_bus;
+    uint32_t pci_device;
+    uint32_t pci_function;
+};
+#endif  // LOADER_ENABLE_LINUX_SORT
+
+// Per enumerated PhysicalDeviceGroup structure, used to wrap in terminator code
+struct loader_physical_device_group_term {
+    struct loader_icd_term *this_icd_term;
+    uint8_t icd_index;
+    VkPhysicalDeviceGroupProperties group_props;
+#ifdef LOADER_ENABLE_LINUX_SORT
+    struct LinuxSortedDeviceInfo internal_device_info[VK_MAX_DEVICE_GROUP_SIZE];
+#endif  // LOADER_ENABLE_LINUX_SORT
+};
+
 struct loader_struct {
     struct loader_instance *instances;
 };
@@ -387,15 +438,10 @@ struct loader_scanned_icd {
 #endif
 };
 
-struct loader_phys_dev_per_icd {
-    uint32_t count;
-    VkPhysicalDevice *phys_devs;
-    struct loader_icd_term *this_icd_term;
-};
-
 enum loader_data_files_type {
-    LOADER_DATA_FILE_MANIFEST_ICD = 0,
-    LOADER_DATA_FILE_MANIFEST_LAYER,
+    LOADER_DATA_FILE_MANIFEST_DRIVER = 0,
+    LOADER_DATA_FILE_MANIFEST_EXPLICIT_LAYER,
+    LOADER_DATA_FILE_MANIFEST_IMPLICIT_LAYER,
     LOADER_DATA_FILE_NUM_TYPES  // Not a real field, used for possible loop terminator
 };
 
@@ -405,7 +451,7 @@ struct loader_data_files {
     char **filename_list;
 };
 
-struct LoaderSortedPhysicalDevice {
+struct loader_phys_dev_per_icd {
     uint32_t device_count;
     VkPhysicalDevice *physical_devices;
     uint32_t icd_index;
@@ -415,4 +461,4 @@ struct LoaderSortedPhysicalDevice {
 struct loader_msg_callback_map_entry {
     VkDebugReportCallbackEXT icd_obj;
     VkDebugReportCallbackEXT loader_obj;
-};
\ No newline at end of file
+};
