@@ -5,13 +5,17 @@
 # SPDX-License-Identifier: Apache-2.0
 
 import argparse
+import os
 import pdb
 import re
 import sys
 import time
 import xml.etree.ElementTree as etree
 
+sys.path.append(os.path.abspath(os.path.dirname(__file__)))
+
 from cgenerator import CGeneratorOptions, COutputGenerator
+
 from docgenerator import DocGeneratorOptions, DocOutputGenerator
 from extensionmetadocgenerator import (ExtensionMetaDocGeneratorOptions,
                                        ExtensionMetaDocOutputGenerator)
@@ -22,12 +26,11 @@ from hostsyncgenerator import HostSynchronizationOutputGenerator
 from formatsgenerator import FormatsOutputGenerator
 from pygenerator import PyOutputGenerator
 from rubygenerator import RubyOutputGenerator
-from reflib import logDiag, logWarn, setLogFile
+from reflib import logDiag, logWarn, logErr, setLogFile
 from reg import Registry
 from validitygenerator import ValidityOutputGenerator
 from apiconventions import APIConventions
 
-
 # Simple timer functions
 startTime = None
 
@@ -40,7 +43,7 @@ def startTimer(timeit):
 
 def endTimer(timeit, msg):
     global startTime
-    if timeit:
+    if timeit and startTime is not None:
         endTime = time.process_time()
         logDiag(msg, endTime - startTime)
         startTime = None
@@ -91,7 +94,7 @@ def makeGenOpts(args):
     # Output target directory
     directory = args.directory
 
-    # Path to generated files, particularly api.py
+    # Path to generated files, particularly apimap.py
     genpath = args.genpath
 
     # Generate MISRA C-friendly headers
@@ -118,7 +121,7 @@ def makeGenOpts(args):
         '/*',
         '** Copyright 2015-2022 The Khronos Group Inc.',
         '**',
-        '** SPDX' + '-License-Identifier: Apache-2.0',
+        '** SPDX-License-Identifier' + ': Apache-2.0',
         '*/',
         ''
     ]
@@ -172,11 +175,11 @@ def makeGenOpts(args):
 
     # Python and Ruby representations of API information, used by scripts
     # that do not need to load the full XML.
-    genOpts['api.py'] = [
+    genOpts['apimap.py'] = [
           PyOutputGenerator,
           DocGeneratorOptions(
             conventions       = conventions,
-            filename          = 'api.py',
+            filename          = 'apimap.py',
             directory         = directory,
             genpath           = None,
             apiname           = defaultAPIName,
@@ -190,11 +193,11 @@ def makeGenOpts(args):
             reparentEnums     = False)
         ]
 
-    genOpts['api.rb'] = [
+    genOpts['apimap.rb'] = [
           RubyOutputGenerator,
           DocGeneratorOptions(
             conventions       = conventions,
-            filename          = 'api.rb',
+            filename          = 'apimap.rb',
             directory         = directory,
             genpath           = None,
             apiname           = defaultAPIName,
@@ -394,7 +397,8 @@ def makeGenOpts(args):
         [ 'vulkan_xlib.h',        [ 'VK_KHR_xlib_surface'         ], commonSuppressExtensions ],
         [ 'vulkan_directfb.h',    [ 'VK_EXT_directfb_surface'     ], commonSuppressExtensions ],
         [ 'vulkan_xlib_xrandr.h', [ 'VK_EXT_acquire_xlib_display' ], commonSuppressExtensions ],
-        [ 'vulkan_metal.h',       [ 'VK_EXT_metal_surface'        ], commonSuppressExtensions ],
+        [ 'vulkan_metal.h',       [ 'VK_EXT_metal_surface',
+                                    'VK_EXT_metal_objects'        ], commonSuppressExtensions ],
         [ 'vulkan_screen.h',      [ 'VK_QNX_screen_surface'       ], commonSuppressExtensions ],
         [ 'vulkan_beta.h',        betaRequireExtensions,             betaSuppressExtensions ],
     ]
@@ -547,7 +551,7 @@ def makeGenOpts(args):
     # but are treated in a similar fashion for generation purposes.
     #
     # Each element of the videoStd[] array is an 'extension' name defining
-    # an iterface, and is also the basis for the generated header file name.
+    # an interface, and is also the basis for the generated header file name.
 
     videoStd = [
         'vulkan_video_codecs_common',
@@ -666,8 +670,6 @@ def genTarget(args):
     # Create generator options with parameters specified on command line
     makeGenOpts(args)
 
-    # pdb.set_trace()
-
     # Select a generator matching the requested target
     if args.target in genOpts:
         createGenerator = genOpts[args.target][0]
@@ -739,8 +741,6 @@ if __name__ == '__main__':
                         help='Use specified registry file instead of vk.xml')
     parser.add_argument('-time', action='store_true',
                         help='Enable timing')
-    parser.add_argument('-validate', action='store_true',
-                        help='Validate the registry properties and exit')
     parser.add_argument('-genpath', action='store', default='gen',
                         help='Path to generated files')
     parser.add_argument('-o', action='store', dest='directory',
@@ -778,10 +778,8 @@ if __name__ == '__main__':
         # Log diagnostics and warnings
         setLogFile(setDiag = True, setWarn = True, filename = '-')
 
-    (gen, options) = (None, None)
-    if not args.validate:
-      # Create the API generator & generator options
-      (gen, options) = genTarget(args)
+    # Create the API generator & generator options
+    (gen, options) = genTarget(args)
 
     # Create the registry object with the specified generator and generator
     # options. The options are set before XML loading as they may affect it.
@@ -797,10 +795,6 @@ if __name__ == '__main__':
     reg.loadElementTree(tree)
     endTimer(args.time, '* Time to parse ElementTree =')
 
-    if args.validate:
-        success = reg.validateRegistry()
-        sys.exit(0 if success else 1)
-
     if args.dump:
         logDiag('* Dumping registry to regdump.txt')
         reg.dumpReg(filehandle=open('regdump.txt', 'w', encoding='utf-8'))
